package org.nuiton.topia;

import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.util.TopiaEntityBinder;
import org.nuiton.topia.persistence.util.TopiaEntityHelper;
import org.nuiton.topia.test.entities.Person;
import org.nuiton.topia.test.entities.Pet;
import org.nuiton.topia.test.entities.Race;
import org.nuiton.topiatest.Address;
import org.nuiton.topiatest.Bill;
import org.nuiton.topiatest.Company;
import org.nuiton.topiatest.Department;
import org.nuiton.topiatest.Employe;
import org.nuiton.topiatest.ExtraDAOEntity;
import org.nuiton.topiatest.NaturalizedEntity;
import org.nuiton.topiatest.Personne;
import org.nuiton.topiatest.Product;
import org.nuiton.topiatest.QueriedEntity;
import org.nuiton.topiatest.Store;
import org.nuiton.topiatest.Type;
import org.nuiton.topiatest.deletetest.Contact2;
import org.nuiton.topiatest.deletetest.Telephone2;
import org.nuiton.util.beans.BinderFactory;
import org.nuiton.util.beans.BinderModelBuilder;

public class TopiaTestBinderHelper extends BinderFactory {

    public static <E extends TopiaEntity> TopiaEntityBinder<E> getTopiaBinder(Class<E> entityClass, String contextName) {
        return (TopiaEntityBinder<E>) newBinder(entityClass, entityClass, contextName, TopiaEntityBinder.class);
    }

    public static <E extends TopiaEntity> TopiaEntityBinder<E> getSimpleTopiaBinder(Class<E> entityClass) {
        return getTopiaBinder(entityClass, "TopiaTest");
    }

    public static void registerTopiaBinder(BinderModelBuilder builder, String contextName) {
       registerBinderModel(builder, contextName);
    }

    public static <E extends TopiaEntity> TopiaEntityBinder<E> registerTopiaBinder(Class<E> entityClass, BinderModelBuilder builder, String contextName) {
       registerBinderModel(builder, contextName);
       return getTopiaBinder(entityClass, contextName);
    }

    public static <E extends TopiaEntity> void copy(String contextName, E source, E target, boolean tech) {
        Class<E> entityClass = (Class<E>) TopiaEntityHelper.getContractClass(TopiaTestDAOHelper.getContracts(), target.getClass());
        TopiaEntityBinder<E> binder = getTopiaBinder(entityClass, contextName);
        if (binder == null) {
            throw new NullPointerException("could not find a simple topia binder of type : " + target.getClass());
        }
        binder.load(source, target, tech);
    }

    public static <E extends TopiaEntity> void simpleCopy(E source, E target, boolean tech) {
        Class<E> entityClass = (Class<E>) TopiaEntityHelper.getContractClass(TopiaTestDAOHelper.getContracts(), target.getClass());
        TopiaEntityBinder<E> binder = getSimpleTopiaBinder(entityClass);
        if (binder == null) {
            throw new NullPointerException("could not find a simple topia binder of type : " + target.getClass());
        }
        binder.load(source, target, tech);
    }

    protected static void initBinders() {
        BinderModelBuilder<Person, Person> builderPerson =
            BinderModelBuilder.newEmptyBuilder(Person.class);
        builderPerson.addSimpleProperties(
            Person.PROPERTY_NAME,
            Person.PROPERTY_FIRSTNAME
        );
        registerTopiaBinder(builderPerson, "TopiaTest");
    
        BinderModelBuilder<Pet, Pet> builderPet =
            BinderModelBuilder.newEmptyBuilder(Pet.class);
        builderPet.addSimpleProperties(
            Pet.PROPERTY_NAME,
            Pet.PROPERTY_TYPE,
            Pet.PROPERTY_PERSON,
            Pet.PROPERTY_RACE
        );
        registerTopiaBinder(builderPet, "TopiaTest");
    
        BinderModelBuilder<Race, Race> builderRace =
            BinderModelBuilder.newEmptyBuilder(Race.class);
        builderRace.addSimpleProperties(
            Race.PROPERTY_NAME
        );
        registerTopiaBinder(builderRace, "TopiaTest");
    
        BinderModelBuilder<Address, Address> builderAddress =
            BinderModelBuilder.newEmptyBuilder(Address.class);
        builderAddress.addSimpleProperties(
            Address.PROPERTY_CITY,
            Address.PROPERTY_ADRESS
        );
        registerTopiaBinder(builderAddress, "TopiaTest");
    
        BinderModelBuilder<Bill, Bill> builderBill =
            BinderModelBuilder.newEmptyBuilder(Bill.class);
        builderBill.addSimpleProperties(
            Bill.PROPERTY_COST,
            Bill.PROPERTY_DATE,
            Bill.PROPERTY_PAID
        );
        registerTopiaBinder(builderBill, "TopiaTest");
    
        BinderModelBuilder<Company, Company> builderCompany =
            BinderModelBuilder.newEmptyBuilder(Company.class);
        builderCompany.addSimpleProperties(
            Company.PROPERTY_NAME,
            Company.PROPERTY_SIRET
        );
        registerTopiaBinder(builderCompany, "TopiaTest");
    
        BinderModelBuilder<Department, Department> builderDepartment =
            BinderModelBuilder.newEmptyBuilder(Department.class);
        builderDepartment.addSimpleProperties(
            Department.PROPERTY_NAME,
            Department.PROPERTY_LEADER,
            Department.PROPERTY_COMPANY
        );
        registerTopiaBinder(builderDepartment, "TopiaTest");
    
        BinderModelBuilder<Employe, Employe> builderEmploye =
            BinderModelBuilder.newEmptyBuilder(Employe.class);
        builderEmploye.addSimpleProperties(
            Employe.PROPERTY_SALARY
        );
        registerTopiaBinder(builderEmploye, "TopiaTest");
    
        BinderModelBuilder<ExtraDAOEntity, ExtraDAOEntity> builderExtraDAOEntity =
            BinderModelBuilder.newEmptyBuilder(ExtraDAOEntity.class);
        builderExtraDAOEntity.addSimpleProperties(
            ExtraDAOEntity.PROPERTY_ATTR1
        );
        registerTopiaBinder(builderExtraDAOEntity, "TopiaTest");
    
        BinderModelBuilder<NaturalizedEntity, NaturalizedEntity> builderNaturalizedEntity =
            BinderModelBuilder.newEmptyBuilder(NaturalizedEntity.class);
        builderNaturalizedEntity.addSimpleProperties(
            NaturalizedEntity.PROPERTY_NATURAL_ID_NOT_NULL,
            NaturalizedEntity.PROPERTY_NATURAL_ID_NULL
        );
        registerTopiaBinder(builderNaturalizedEntity, "TopiaTest");
    
        BinderModelBuilder<Personne, Personne> builderPersonne =
            BinderModelBuilder.newEmptyBuilder(Personne.class);
        builderPersonne.addSimpleProperties(
            Personne.PROPERTY_NAME,
            Personne.PROPERTY_GENDER,
            Personne.PROPERTY_OTHER_GENDER,
            Personne.PROPERTY_ADDRESS
        );
        registerTopiaBinder(builderPersonne, "TopiaTest");
    
        BinderModelBuilder<Product, Product> builderProduct =
            BinderModelBuilder.newEmptyBuilder(Product.class);
        builderProduct.addSimpleProperties(
            Product.PROPERTY_NAME,
            Product.PROPERTY_TYPE
        );
        registerTopiaBinder(builderProduct, "TopiaTest");
    
        BinderModelBuilder<QueriedEntity, QueriedEntity> builderQueriedEntity =
            BinderModelBuilder.newEmptyBuilder(QueriedEntity.class);
        builderQueriedEntity.addSimpleProperties(
            QueriedEntity.PROPERTY_TEST_ADD
        );
        registerTopiaBinder(builderQueriedEntity, "TopiaTest");
    
        BinderModelBuilder<Store, Store> builderStore =
            BinderModelBuilder.newEmptyBuilder(Store.class);
        builderStore.addSimpleProperties(
            Store.PROPERTY_NAME
        );
        registerTopiaBinder(builderStore, "TopiaTest");
    
        BinderModelBuilder<Type, Type> builderType =
            BinderModelBuilder.newEmptyBuilder(Type.class);
        builderType.addSimpleProperties(
            Type.PROPERTY_NAME
        );
        registerTopiaBinder(builderType, "TopiaTest");
    
        BinderModelBuilder<Contact2, Contact2> builderContact2 =
            BinderModelBuilder.newEmptyBuilder(Contact2.class);
        builderContact2.addSimpleProperties(
            Contact2.PROPERTY_CONTACT_VALUE,
            Contact2.PROPERTY_TYPE
        );
        registerTopiaBinder(builderContact2, "TopiaTest");
    
        BinderModelBuilder<Telephone2, Telephone2> builderTelephone2 =
            BinderModelBuilder.newEmptyBuilder(Telephone2.class);
        builderTelephone2.addSimpleProperties(
            Telephone2.PROPERTY_PREFIX,
            Telephone2.PROPERTY_COUNTRY
        );
        registerTopiaBinder(builderTelephone2, "TopiaTest");
    }

    static {
    
    initBinders();

    }

} //TopiaTestBinderHelper
