package org.nuiton.topiatest.deletetest;

import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.nuiton.i18n.I18n;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.persistence.EntityVisitor;
import org.nuiton.topia.persistence.TopiaEntity;

public abstract class Telephone2Abstract extends Contact2Impl implements Telephone2 {

    /**
     * Nom de l'attribut en BD : prefix
     */
    protected String prefix;

    /**
     * Nom de l'attribut en BD : country
     */
    protected String country;

    private static final long serialVersionUID = 3616498692672139876L;

    static {
    
        I18n.n_("topia.test.common.telephone2");
        I18n.n_("topia.test.common.prefix");
        I18n.n_("topia.test.common.country");
    }

    @Override
    public void accept(EntityVisitor visitor) throws TopiaException {
        visitor.start(this);
        visitor.visit(this, PROPERTY_PREFIX, String.class, prefix);
        visitor.visit(this, PROPERTY_COUNTRY, String.class, country);
        visitor.end(this);
    }

    @Override
    public void setPrefix(String prefix) {
        String oldValue = this.prefix;
        fireOnPreWrite(PROPERTY_PREFIX, oldValue, prefix);
        this.prefix = prefix;
        fireOnPostWrite(PROPERTY_PREFIX, oldValue, prefix);
    }

    @Override
    public String getPrefix() {
        fireOnPreRead(PROPERTY_PREFIX, prefix);
        String result = this.prefix;
        fireOnPostRead(PROPERTY_PREFIX, prefix);
        return result;
    }

    @Override
    public void setCountry(String country) {
        String oldValue = this.country;
        fireOnPreWrite(PROPERTY_COUNTRY, oldValue, country);
        this.country = country;
        fireOnPostWrite(PROPERTY_COUNTRY, oldValue, country);
    }

    @Override
    public String getCountry() {
        fireOnPreRead(PROPERTY_COUNTRY, country);
        String result = this.country;
        fireOnPostRead(PROPERTY_COUNTRY, country);
        return result;
    }

    @Override
    public List<TopiaEntity> getAggregate() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            result.add(entity);
            result.addAll(entity.getAggregate());
        }
        return result;
    }

    @Override
    public List<TopiaEntity> getComposite() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            if (entity != null) {
                result.add(entity);
                result.addAll(entity.getComposite());
            }
        }
        return result;
    }

    @Override
    public String toString() {
        String result = new ToStringBuilder(this).
            append(PROPERTY_PREFIX, this.prefix).
            append(PROPERTY_COUNTRY, this.country).
         toString();
        return result;
    }

} //Telephone2Abstract
